package com.sun.org.apache.xalan.internal.xsltc.compiler;

import com.sun.org.apache.bcel.internal.generic.InstructionList;
import com.sun.org.apache.xalan.internal.xsltc.compiler.util.MethodGenerator;
import com.sun.org.apache.xalan.internal.xsltc.compiler.util.ClassGenerator;
import com.sun.org.apache.xalan.internal.xsltc.compiler.util.TypeCheckError;
import com.sun.org.apache.xalan.internal.xsltc.compiler.util.MethodType;
import com.sun.org.apache.xalan.internal.xsltc.compiler.util.Type;

final class UnaryOpExpr extends Expression
{
    private Expression _left;
    
    public UnaryOpExpr(Expression left) {
        (this._left = left).setParent(this);
    }
    
    @Override
    public boolean hasPositionCall() {
        return this._left.hasPositionCall();
    }
    
    @Override
    public boolean hasLastCall() {
        return this._left.hasLastCall();
    }
    
    public void setParser(Parser parser) {
        super.setParser(parser);
        this._left.setParser(parser);
    }
    
    @Override
    public Type typeCheck(SymbolTable stable) throws TypeCheckError {
        Type tleft = this._left.typeCheck(stable);
        MethodType ptype = this.lookupPrimop(stable, "u-", new MethodType(Type.Void, tleft));
        if (ptype != null) {
            Type arg1 = (Type)ptype.argsType().get(0);
            if (!arg1.identicalTo(tleft)) {
                this._left = new CastExpr(this._left, arg1);
            }
            return this._type = ptype.resultType();
        }
        else {
            throw new TypeCheckError(this);
        }
    }
    
    @Override
    public String toString() {
        return "u-(" + this._left + 41;
    }
    
    @Override
    public void translate(ClassGenerator classGen, MethodGenerator methodGen) {
        InstructionList il = methodGen.getInstructionList();
        this._left.translate(classGen, methodGen);
        il.append(this._type.NEG());
    }
}
